<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;

use Ems\Services\ControleEntregaEPIService;
class ControleEntregaEPIController {

    /** @var ControleEntregaEPIService */
    protected $controleEntregaEPIService;

    public function __construct(
        EntityManager $entityManager,
        ControleEntregaEPIService $controleEntregaEPIService
    ) {
        $this->entityManager             = $entityManager;
        $this->controleEntregaEPIService = $controleEntregaEPIService;
    }

    public function getControleEntregaEPI(
        Request\Row $request,
        Response $response
    ) {
        try {
            $params = $request->getRow();
            
            $nrestruturam       = $params['NRESTRUTURAM'];
            $nrperiodoapuracao  = $params['NRPERIODOAPURACAO'];
            
            $controleEntregaEPI = $this->controleEntregaEPIService->getControleEntregaEPI($nrestruturam, $nrperiodoapuracao);
            $response->addDataSet(new DataSet('getControleEntregaEPI', $controleEntregaEPI));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getEpi(Request\Row $request, DTO\Response $response) {
        try {
            $params = $request->getRow();
            $nrorg = $params['NRORG'];

            $epi = $this->controleEntregaEPIService->getEpi($nrorg);
            $response->addDataSet(new DataSet('getEpi', $epi));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function setHistoricoEpi(Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow()->getArrayCopy();

            $success = $this->controleEntregaEPIService->setHistoricoEpi($row);

            $response->addDataSet(new DataSet('setHistoricoEpi', array('success' => $success)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteHistoricoEpi(Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow()->getArrayCopy();

            $success = $this->controleEntregaEPIService->deleteHistoricoEpi($row);

            $response->addDataSet(new DataSet('deleteHistoricoEpi', array('success' => $success)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}