<?php

namespace Ems\Util;

use Doctrine\DBAL\Connection;

/**
 *  Abstract Repository
 * 
 *  Define rotinas comuns as entidades 'Repository'
 * 
 */

abstract class Repository {
    private const ENTITY_MANAGER_NAME = 'entityManager';

    /**
     *  Rotina responsavel por obter a entidade capaz de realizar as consultas no banco de dados.
     * 
     *  @return Connection
     */
    public function getConnection(): Connection {
        return Utils::getInstance(self::ENTITY_MANAGER_NAME)->getConnection();
    }

    /**
     *  Rotina responsavel por realizar a consulta.
     * 
     *  @param  string  $sql
     *  @param  array   $paramters
     */
    public function get(
        string  $sql,
        array   $paramters
    ) {
        $query = $this->getConnection()->prepare($sql);
        $query->execute($paramters);

        return $query->fetchAll();
    }
}