<?php

namespace EMS\Util;

use Ems\Util\FileServer;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Util\DateUtil;

use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Utilities {

    /** @var FileServer */
    protected $fileServer;

    public const SPREADSHEET_MIME_TYPE = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet';
    public const SPREADSHEET_FILE_EXTENSION = 'xlsx';

    public function __construct(
        FileServer $fileServer
    ) {
        $this->fileServer = $fileServer;
    }

    public static function getRemoteFileSize($url) {
        $data = get_headers($url, true);
        if (isset($data['Content-Length'])) {
            return (int)$data['Content-Length'];
        }
        return null;
    }

    public static function downloadAndRemoveFile($fileDir, $fileName, $fileType = 'text/plain', $isRemoveFile = true) {
        if(strpos($fileDir, 'http') === false) {
            $fileSize = filesize($fileDir);
        } else {
            $fileSize = self::getRemoteFileSize($fileDir);
            $header   = array (
                "apiKey: " . self::getFileServerApikey()
            );
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $fileDir);
            curl_setopt($ch, CURLOPT_POST, false);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
            curl_setopt($ch, CURLOPT_TIMEOUT, 30);

            $resultado = curl_exec($ch);
            $error = curl_error($ch);
            curl_close($ch);

            if (!empty($error)) {
                throw new \Exception($error);
            } else {
                // $fileDir = /*self::getTempLogPathParameter() .*/ DIRECTORY_SEPARATOR . $fileName;
                $fileDir = '/tmp' . DIRECTORY_SEPARATOR . $fileName;
                var_dump($resultado);die;
                file_put_contents($fileDir, $resultado);
            }
        }

        header('Accept-Ranges: none');
        header("Content-Type: " . $fileType);
        header("Content-Length: " . $fileSize);
        header("Content-Disposition: attachment; filename=" . $fileName);
        readfile($fileDir);

        if($isRemoveFile) {
            unlink($fileDir);
        }
        exit;
    }

    public static function formatDate(string $date): string {
        $formattedDate = explode(' ', $date)[0];

        return $formattedDate;
    }

    public function saveFileOnFileServer(
        $b64FileContent,
        $fileExtension,
        $nrorg,
        $fileDir = null,
        $fileName = null,
        $customFolder = '',
        $marcacao = true
    ): string {
        try {
            if ($fileName === null) {
                $fileName = $this->geraNomeArquivo($fileExtension, $marcacao);
            }

            if ($fileDir === null) {
                $fileDir = $this->geraDiretorioArquivo($fileExtension, $nrorg, $marcacao);
            }

            $type = self::getFileMIMEType($b64FileContent);

            // METODO NOVO USANDO CLASSES PRÓPRIAS
            $params = [
                "name"               => $fileName, // "documento_ocorrencia_07_07_2020_12_53_05.jpeg"
                "folderName"         => $fileDir, // "/EMS/20/Fotos/"
                "b64File"            => $b64FileContent, // "data:image/jpeg;base64,......."
                "type"               => $type, // "image/jpeg"
                "size"               => 1024,
                "path"               => $fileDir // "/EMS/20/Fotos/"
            ];


            $path = $this->fileServer->upload($params);

            return $path;

            // METODO ANTIGO, UTILIZANDO O FILEUPLOADER
            // $fileFullDir = $fileDir . $fileName . "." . $fileExtension;
            // $lastModifiedDate = DateUtil::getDataAtual()->format(DateUtil::FORMATO_AMERICANO_DATAHORA);
            // $lastModified = 1507122512763;
            // $fileSize = strlen($b64FileContent);
            // $fileType = "application/" . $fileExtension;
            // $webkitRelativePath = "";
            // $fileFieldName = "DSPATH";

            // $row = array(
            //     "DSPATH" => array(
            //         array(
            //             // "name"               => "/EMS/Fotos/20/". $fileName . "." . $fileType,
            //             "name"               => $fileFullDir,
            //             "lastModified"       => $lastModified,
            //             "lastModifiedDate"   => $lastModifiedDate,
            //             "webkitRelativePath" => $webkitRelativePath,
            //             "size"               => $fileSize,
            //             "type"               => $fileType,
            //             "b64File"            => $b64FileContent
            //     )
            // );

            // $this->zhFileUploader->uploadFilesInRow($row, $fileFieldName, $customFolder);
            // $filePath = $row['DSPATH'][0]['path'];
            // return $filePath;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public static function geraNomeArquivo(
        string $fileType,
        $marcacao = true
    ): string {
        $date = new \DateTime();

        if ($marcacao) {
            return 'foto_marcacao_' . $date->format('d_m_Y_H_i_s');
        } else {
            return 'documento_ocorrencia_' . $date->format('d_m_Y_H_i_s');
        }
    }

    public function geraDiretorioArquivo(
        string $fileType,
        int $nrorg,
        $marcacao = true
    ): string {
        if ($marcacao) {
            switch ($fileType) {
                case 'jpeg':
                    return "/EMS/$nrorg/Fotos/";
            }
        } else {
            return "/EMS/$nrorg/Fotos/";
        }
    }

    public function getFileServerApikey() : string {
        return "ea1999f0a10e053ae87d08e2691f65f4";
    }

    public function deleteFile(string $path): bool {
        $deleted = $this->fileServer->delete($path);

        return $deleted;
    }

    public function downloadFile(string $path): array {
        $file = $this->fileServer->download($path);

        return $file;
    }

    public static function getFileMIMEType(string $base64): string {
        $part1 = explode(';', $base64)[0];
        $type = explode(':', $part1)[1];

        return $type;
    }
    
    public static function getBase64Content(string $base64) {
        $base64ContentParts = explode(',', $base64);
        if (count($base64ContentParts) !== 2) {
            return false;
        }
        
        return $base64ContentParts[1];
    } 

    public static function blobToBase64(string $blob): array {
        $base64 = base64_encode($blob);

        $f = finfo_open();
        $mime_type = finfo_buffer($f, $blob, FILEINFO_MIME_TYPE);

        $file = [
            'type'   => $mime_type,
            'base64' => $base64
        ];

        return $file;
    }
    
    public static function blobToFile(string $base64Content, string $fileName): string {
        $fileContentEncoded = self::getBase64Content($base64Content);
        $fileContent = base64_decode($fileContentEncoded);
        
        $fullPath = self::getTempDirectory() . DIRECTORY_SEPARATOR . $fileName;
        
        file_put_contents($fullPath, $fileContent);
        return $fullPath;
    }
    
    public static function getTempDirectory(): string {
        return InstanceManager::getInstance()->getParameter('temp_directory');
    }
}


/*
string(252) "{"dataset":{"files":[{"FILE":[{"orgId":{"$oid":"5888de55b47f44fc17000029"},"NAME":"\/EMS\/Fotos\/20\/foto_marcacao_27_12_2019.jpeg","TYPE":"image\/jpeg","SIZE":55407,"DATE":"30\/12\/2019 11:16:34","PATH":"tkOrg205888de555f18e\/5e0a06c264175.jpeg"}]}]}}"
*/